"""Contains classes for all sorts of objects (TOTALLY ripped from Unity)."""

from pyglet.sprite import Sprite

from stargateRL.utils import GX_TILESETS
from stargateRL.engine.graphx import TileColor


class SpriteObject(object):
    """Spriteobject are objects made out of sprites."""

    def __init__(self, batch=None, group_order=None):
        """Construct sprite object."""
        pass


class SpriteText(SpriteObject):
    """This can be a single word, a line of text or even a block of text."""

    def __init__(self, string, x, y, x_tiles=None, y_tiles=None,
                 tile_color=TileColor(), batch=None, group_order=None):
        """Construct the text out of sprites."""
        self._string = string
        self._batch = batch
        self._group_order = group_order

        # Create sprites from string
        ts = GX_TILESETS['MAIN'].tile_size
        self._sprites = []
        (x_mod, y_mod) = (0, 0)
        for letter in string:
            letter_ord = ord(letter)
            if letter_ord == 10:
                y_mod -= 1
                x_mod = 0
                continue
            elif x_tiles is not None and x_mod >= x_tiles:
                y_mod -= 1
                x_mod = 0

            x_mod += 1
            self._sprites.append(
                Sprite(
                    GX_TILESETS['MAIN'].get_colored(letter_ord, tile_color),
                    (x+x_mod)*ts, (y+y_mod)*ts,
                    batch=self._batch, group=self._group_order))

    def set_color(self, tile_color=TileColor(), start=0, end=None):
        """Change the color of the text or part of the text."""
        for index, sprite in enumerate(self._sprites[start:end]):
            sprite.image =\
                GX_TILESETS['MAIN'].get_colored(ord(self._string[index+start]),
                                                tile_color)

    def set_batch(self, batch=None):
        """Set the batch of the sprites."""
        for sprite in self._sprites:
            sprite.batch = batch

    def set_group(self, group_order=None, start=0, end=None):
        """Set the group of the sprites."""
        for sprite in self._sprites[start:end]:
            sprite.group = group_order



"""Main method for stargateRL."""
import pyglet

from stargateRL.engine.screen import GameWindow
from stargateRL.engine.graphx import TileColor
from stargateRL.engine import widgets
from stargateRL.utils import CONFIG

# Create the game window
window_config = CONFIG['window']

window = GameWindow(window_config['width'],
                    window_config['height'],
                    fullscreen=window_config['fullscreen'],
                    resizable=window_config['resizable'],
                    style=window_config['style'])

# Configurate window
window.set_mouse_visible(window_config['mouse'])
window.set_icon(pyglet.resource.image(CONFIG['resources']['icon']))

# Render screen border
screen_border = widgets.BorderWidget(0, 0, window.x_tiles, window.y_tiles,
                                     TileColor('border', 'transparent'))

selection_menu =\
    widgets.SelectionMenuWidget(window.x_tiles / 4, window.y_tiles / 2,
                                window.x_tiles / 2, window.y_tiles / 4,
                                TileColor('gold', 'transparent'),
                                TileColor('white', 'transparent'),
                                TileColor('red', 'gold'),
                                'Settings', 'Exit')

window.push_widget(screen_border)
window.push_widget(selection_menu)

pyglet.app.run()



"""The main launcher for changing the configurations."""


import json
import os
import tkinter as tk

from stargateRL.launcher import elements


def launch():
    """Start the stargateRL.__main__ method."""
    root.destroy()
    import stargateRL.__main__


def load_config():
    """Load the config file."""
    config_file_name = os.path.abspath(
        os.path.join(os.path.dirname(__file__),
                     os.pardir, os.pardir, 'config.json'))
    with open(config_file_name, 'r') as config_file:
        return json.load(config_file)


def save_config(config_dictionary):
    """Save the config changes."""
    config_file_name = 'config.json'
    if os.path.isfile('config.local.json'):
        config_file_name = 'config.local.json'
    with open(config_file_name, 'w') as config_file:
        json.dump(config_dictionary, config_file, indent=4)


def save():
    """Method called by the Apply button."""
    config_dictionary = CONFIG
    for section in CONFIG.keys():
        for key, value in CONFIG[section].items():
            config_dictionary[section][key] =\
                config_frame.subframes[section].options[key].get_value()
    save_config(config_dictionary)


def default():
    """Change the configs back to the default."""
    pass


def close():
    """Method called by the Close button."""
    popup = tk.Toplevel(master=None)
    popup.title('Are you sure?')
    popup.resizable(0, 0)

    popup.grab_set()

    label = tk.Label(popup, anchor=tk.N,
                     text='Are you sure you want to exit without saving?')
    label.pack(side=tk.TOP, fill=tk.X)

    tk.Button(popup, anchor=tk.CENTER, text='Yes',
              command=root.destroy).pack(side=tk.LEFT)
    tk.Button(popup, anchor=tk.CENTER, text='No',
              command=popup.destroy).pack(side=tk.LEFT)

    popup.mainloop()
    popup.destroy()


root = tk.Tk()
root.resizable(0, 0)
root.wm_title('stargateRL Launcher')

CONFIG = load_config()

# GUI goes bellow this line
# Generate the sections
config_frame = elements.MainFrame(root)
config_frame.make_subframes(CONFIG.keys())

# Fill the sections with options
for section in CONFIG.keys():
    for key, value in CONFIG[section].items():
        value_type = type(value)
        if value_type == int:
            config_frame.subframes[section].options[key] =\
                elements.IntegerInput(
                    config_frame.subframes[section], key, value)
        elif value_type == bool:
            config_frame.subframes[section].options[key] =\
                elements.BooleanInput(
                    config_frame.subframes[section], key, value)
        elif value_type == str or value_type == unicode:
            config_frame.subframes[section].options[key] =\
                elements.StringInput(
                    config_frame.subframes[section], key, value)

tk.Button(config_frame, highlightbackground=elements.MAINFRAME_COLORS[0],
          text='Apply', command=save).pack(side=tk.LEFT, fill=tk.X)
tk.Button(config_frame, highlightbackground=elements.MAINFRAME_COLORS[0],
          text='Cancel', command=close).pack(side=tk.LEFT, fill=tk.X)
tk.Button(config_frame, highlightbackground=elements.MAINFRAME_COLORS[0],
          text='Default', command=default).pack(side=tk.LEFT, fill=tk.X)

tk.Button(config_frame, highlightbackground=elements.MAINFRAME_COLORS[0],
          text='Launch', command=launch).pack(side=tk.RIGHT, fill=tk.X)

# GUI goes above this line
root.mainloop()
root.destroy()



"""Graphical resource manager."""

import pyglet


class Color(object):
    """A representation of a color. RGBA."""

    def __init__(self, r, g, b, a):
        """Create a color."""
        self._color = (r, g, b, a)

    def __call__(self):
        """Return the color only."""
        return bytes(bytearray(self._color))


class TileColor(object):
    """A class holding two colors (foreground and background)."""

    def __init__(self, foreground_color='white',
                 background_color='transparent'):
        """Construct the color for the tile."""
        self._foreground_color = foreground_color
        self._background_color = background_color

    def __call__(self):
        """Return the names of the colors."""
        return (self._foreground_color, self._background_color)

    def set_background(self, color):
        """Set the background color."""
        self._background_color = color

    def set_foreground(self, color):
        """Set the foreground color."""
        self._foreground_color = color

    def get_background(self):
        """Return the background color."""
        return self._background_color

    def get_foreground(self):
        """Return the foreground color."""
        return self._foreground_color


COLORS = {'black': Color(0, 0, 0, 255), 'white': Color(255, 255, 255, 255),
          'blue': Color(0, 0, 255, 255), 'red': Color(255, 0, 0, 255),
          'green': Color(0, 255, 0, 255), 'transparent': Color(0, 0, 0, 0),
          'border': Color(70, 76, 84, 255), 'gold': Color(245, 211, 115, 255)}


class GxTileset(object):
    """A image grid composing the Tileset resource."""

    def __init__(self, resource_path, tile_size):
        """Construct the tileset manager."""
        self.source_image = pyglet.resource.image(resource_path)

        self.tile_size = int(tile_size)
        self.x_tiles_count = int(self.source_image.width / tile_size)
        self.y_tiles_count = int(self.source_image.height / tile_size)

        self.tileset_grid = pyglet.image.ImageGrid(self.source_image,
                                                   self.x_tiles_count,
                                                   self.y_tiles_count)

        # Normalize Pyglet's strange way of indexing ImageGrids
        self.tileset_grid = \
            [self.tileset_grid[x:x+self.tile_size]
             for x in range(0, len(self.tileset_grid), self.tile_size)]
        self.tileset_grid = self.tileset_grid[::-1]

    def get_by_id(self, tile_id):
        """Return a Image from the ImageGrid of the tileset."""
        row = self.tileset_grid[int(tile_id / self.tile_size)]
        return row[tile_id % self.tile_size]

    def get(self, row, column):
        """Return the Image from x, y inside ImageGrid."""
        return self.tileset_grid[row][column]

    def get_colored(self, tile_id, tile_color):
        """Return a tile with the given colors."""
        tile_image = self.get_by_id(tile_id)
        image_data = tile_image.image_data.get_data('RGBA', tile_image.width*4)
        image_pxls = [image_data[p:p+4] for p in range(0, len(image_data), 4)]

        image_background = [p for p in range(len(image_pxls))
                            if image_pxls[p] == COLORS['black']()]
        image_foreground = [p for p in range(len(image_pxls))
                            if image_pxls[p] == COLORS['white']()]

        for pixel in image_background:
            image_pxls[pixel] = COLORS[tile_color.get_background()]()
        for pixel in image_foreground:
            image_pxls[pixel] = COLORS[tile_color.get_foreground()]()

        combined_pixels = b''
        for pixel in image_pxls:
            combined_pixels += pixel

        return pyglet.image.ImageData(tile_image.width, tile_image.height,
                                      'RGBA', combined_pixels)



"""Window/Screen manager."""

import pyglet

from stargateRL.utils import GX_TILESETS


class GameWindow(pyglet.window.Window):
    """Window manager."""

    def __init__(self, width, height, **kargs):
        """Construct the window manager."""
        super(GameWindow, self).__init__(width, height, **kargs)
        self._widgets = []
        self.fps = pyglet.clock.ClockDisplay()

        self.x_tiles = int(width/GX_TILESETS['MAIN'].tile_size)
        self.y_tiles = int(height/GX_TILESETS['MAIN'].tile_size)

        # DEBUG
        self._batch = pyglet.graphics.Batch()

    def push_widget(self, widget):
        """Put a widget on top of the widget stack."""
        self._widgets.append(widget)

    def pop_widget(self):
        """Pop the widget on top of the widget stack."""
        if self._widgets:
            self._widgets.pop()

    # Window events

    def on_resize(self, width, height):
        """Pyglet on window size change."""
        super(GameWindow, self).on_resize(width, height)

    def on_draw(self):
        """Pyglet Window draw method."""
        self.clear()
        for widget in self._widgets:
            widget.draw()

        # DEBUG
        self._batch.draw()

        self.fps.draw()

    def on_key_press(self, symbol, modifiers):
        """Whenever a key is pressed (there is also a release method)."""
        if self._widgets:
            if not self._widgets[-1].on_key_press(symbol, modifiers):
                self.pop_widget()
        else:
            super(GameWindow, self).on_key_press(symbol, modifiers)



"""Contains useful methods and global variables."""
import os
import json
import pyglet

from stargateRL.engine.graphx import GxTileset

__all__ = ['CONFIG', 'GX_TILESETS']

# Define resource path
ROOT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir))
pyglet.resource.path = [(os.path.abspath(os.path.join(ROOT_DIR, 'bin')))]


def load_config():
    """Load the config file."""
    config_file_name = os.path.abspath(
        os.path.join(os.path.dirname(__file__), os.pardir, 'config.json'))
    with open(config_file_name, 'r') as config_file:
        return json.load(config_file)


# Load the config
CONFIG = load_config()
# Load the tileset
GX_TILESETS = {'MAIN': GxTileset(CONFIG['resources']['tileset'],
                                 CONFIG['resources']['size'])}



"""A set of GUI elements for the launcher."""

import tkinter as tk
import tkinter.font as tkFont


# First color is the background, second color is the foreground
HEADER_COLORS = ('#37343E', '#F5EDEA')
VALUE_COLORS = ('', '#F5EDEA')
MAINFRAME_COLORS = ('#37343E', '')
SUBFRAME_COLORS = ('#7E7F7F', '')


class MainFrame(tk.Frame):
    """The main frame element."""

    def __init__(self, master):
        """Construct the main frame."""
        tk.Frame.__init__(self, master, bg=MAINFRAME_COLORS[0])
        self.pack(fill=tk.BOTH)

        self.subframes = {}

    def make_subframes(self, frames):
        """Generate empty subframes for the settings."""
        for frame in frames:
            self.subframes[frame] = SubFrame(self, frame)


class SubFrame(tk.Frame):
    """A section of settings."""

    def __init__(self, master, label):
        """Construct the sub frame."""
        tk.Frame.__init__(self, master, bg=SUBFRAME_COLORS[0])
        self.pack(side=tk.TOP, fill=tk.BOTH)
        self.options = {}

        header_font = tkFont.Font(family='Serif', size=18, weight=tkFont.BOLD)
        self._label = tk.Label(master=self, text='# '+label, anchor=tk.NW,
                               padx=25, font=header_font,
                               bg=HEADER_COLORS[0], fg=HEADER_COLORS[1])
        self._label.pack(fill=tk.X)


class Input(tk.Frame, object):
    """Standards for a input."""

    def __init__(self, master, label):
        """Construct standard input."""
        tk.Frame.__init__(self, master, bg=SUBFRAME_COLORS[0])
        self.pack(side=tk.TOP, fill=tk.X)

        self._label = tk.Label(master=self, text=label+':', anchor=tk.W,
                               padx=30, width=10, bg=SUBFRAME_COLORS[0])
        self._label.pack(fill=tk.X, side=tk.LEFT)

        self._value = None

    def get_value(self):
        """Return the value from the input."""
        return self._value


class IntegerInput(Input):
    """Input a integer."""

    def __init__(self, master, label, value):
        """Construct the integer input."""
        super(IntegerInput, self).__init__(master, label)

        self._value = tk.StringVar(value=value)
        self._entry = tk.Entry(master=self, textvariable=self._value,
                               bg=SUBFRAME_COLORS[0], relief=tk.FLAT,
                               highlightbackground=SUBFRAME_COLORS[0],
                               fg=VALUE_COLORS[1])
        self._entry.pack(fill=tk.X, side=tk.RIGHT)

    def get_value(self):
        """Return the value from the entry."""
        return int(self._entry.get())


class BooleanInput(Input):
    """Input a boolean."""

    def __init__(self, master, label, value):
        """Construct the boolean input."""
        super(BooleanInput, self).__init__(master, label)

        self._value = tk.BooleanVar(value=value)
        self._entry = tk.Checkbutton(master=self, variable=self._value,
                                     onvalue=True, offvalue=False,
                                     bg=SUBFRAME_COLORS[0])
        self._entry.pack(fill=tk.X, side=tk.LEFT)

    def get_value(self):
        """Return the value from the entry."""
        return self._value.get()


class StringInput(Input):
    """Input a string."""

    def __init__(self, master, label, value):
        """Construct the string input."""
        super(StringInput, self).__init__(master, label)

        self._value = tk.StringVar(value=value)
        self._entry = tk.Entry(master=self, textvariable=self._value,
                               bg=SUBFRAME_COLORS[0], relief=tk.FLAT,
                               highlightbackground=SUBFRAME_COLORS[0],
                               fg=VALUE_COLORS[1])
        self._entry.pack(fill=tk.X, side=tk.RIGHT)

    def get_value(self):
        """Return the value from the input."""
        return self._value.get()



"""Widget support."""

from pyglet.sprite import Sprite
from pyglet.graphics import Batch
from pyglet.window import key as wkey

from stargateRL.engine.objects import SpriteText
from stargateRL.utils import GX_TILESETS


class Widget(object):
    """Standard widget with requierments."""

    def __init__(self):
        """Standard widget construct."""
        self._batch = Batch()
        self._sprites = []

    def draw(self):
        """Draw the pyglet batch."""
        self._batch.draw()

    def on_key_press(self, symbol, modifiers):
        """Returning false removes the widget from stack, true keeps it."""
        return False


class FilledBoxWidget(Widget):
    """A widget that fills the screen with colored blocks."""

    def __init__(self, x, y, x_tiles, y_tiles, tile_color, tile_id=177):
        """Construct a box at given position of given dimensions w color."""
        super(FilledBoxWidget, self).__init__()
        tile_size = GX_TILESETS['MAIN'].tile_size
        for x_tile in range(x_tiles):
            for y_tile in range(y_tiles):
                self._sprites.append(
                    Sprite(
                        GX_TILESETS['MAIN'].get_colored(tile_id, tile_color),
                        (x + x_tile) * tile_size,
                        (y + y_tile) * tile_size,
                        batch=self._batch))


class BorderWidget(Widget):
    """A widget that draws a widget around the given area."""

    def __init__(self, x, y, x_tiles, y_tiles, tile_color, **kargs):
        """Construct a border widget, can be used with other widgets."""
        super(BorderWidget, self).__init__()
        ts = GX_TILESETS['MAIN'].tile_size

        # TODO: better way of defining border tiles
        LT_CORNER = kargs.get('LT_CORNER', 178)
        LB_CORNER = kargs.get('LB_CORNER', 178)
        RB_CORNER = kargs.get('RB_CORNER', 178)
        RT_CORNER = kargs.get('RT_CORNER', 178)

        T_EDGE = kargs.get('T_EDGE', 35)
        B_EDGE = kargs.get('B_EDGE', 35)
        L_EDGE = kargs.get('L_EDGE', 35)
        R_EDGE = kargs.get('R_EDGE', 35)

        x_tiles = int(x_tiles)
        y_tiles = int(y_tiles)

        # Left, bottom, corner
        self._sprites.append(
            Sprite(
                GX_TILESETS['MAIN'].get_colored(LB_CORNER, tile_color),
                x * ts, y * ts, batch=self._batch))
        # Left, top, corner
        self._sprites.append(
            Sprite(
                GX_TILESETS['MAIN'].get_colored(LT_CORNER, tile_color),
                x * ts, (y + y_tiles - 1) * ts, batch=self._batch))
        # Right, bottom, corner
        self._sprites.append(
            Sprite(
                GX_TILESETS['MAIN'].get_colored(RB_CORNER, tile_color),
                (x + x_tiles - 1) * ts, y * ts, batch=self._batch))
        # Right, top, corner
        self._sprites.append(
            Sprite(
                GX_TILESETS['MAIN'].get_colored(RT_CORNER, tile_color),
                (x + x_tiles - 1) * ts, (y + y_tiles - 1) * ts,
                batch=self._batch))

        # Bottom and top edges
        for x_tile in range(x_tiles):
            self._sprites.append(
                Sprite(
                    GX_TILESETS['MAIN'].get_colored(T_EDGE, tile_color),
                    (x + x_tile) * ts, (y + y_tiles - 1) * ts,
                    batch=self._batch))
            self._sprites.append(
                Sprite(
                    GX_TILESETS['MAIN'].get_colored(B_EDGE, tile_color),
                    (x + x_tile) * ts, y * ts, batch=self._batch))
        # Left and right edges
        for y_tile in range(y_tiles):
            self._sprites.append(
                Sprite(
                    GX_TILESETS['MAIN'].get_colored(R_EDGE, tile_color),
                    (x + x_tiles - 1) * ts, (y + y_tile) * ts,
                    batch=self._batch))
            self._sprites.append(
                Sprite(
                    GX_TILESETS['MAIN'].get_colored(L_EDGE, tile_color),
                    x * ts, (y + y_tile) * ts,
                    batch=self._batch))

    def on_key_press(self, symbol, modifiers):
        """Get called on window.event on_key_press."""
        return True


# TODO: SelectionMenu should have some sort of way performing an action
# when `wkey.RETURN` is pressed on the current active option `self._active`
# Possibly a dictionary of methods? Where the keys are integers?
# eg: self.ACTIONS[self._active] %params%
# This would have another method to set the ACTIONS
class SelectionMenuWidget(BorderWidget):
    """A menu of options."""

    def __init__(self, x, y, x_tiles, y_tiles,
                 border_color, tile_color, active_color,
                 *options, **kargs):
        """Construct the selection menu."""
        super(SelectionMenuWidget, self).__init__(x, y, x_tiles, y_tiles,
                                                  border_color, **kargs)

        # Store options and active option
        self._options = options
        self._active = 0
        self._text_objects = []
        self._colors = (tile_color, active_color)

        op_lenght = len(options)
        for index, option in enumerate(options):
            word_length = len(option)
            self._text_objects.append(
                SpriteText(option, x+x_tiles/2-word_length/2,
                           y+y_tiles/2-index+op_lenght,
                           None, None, tile_color, self._batch))
        self._text_objects[self._active].set_color(self._colors[1])

    def on_key_press(self, symbol, modifiers):
        """Get called on window.event on_key_press."""
        if symbol == wkey.UP:
            if self._active <= 0:
                self._text_objects[self._active].set_color(self._colors[0])
                self._active = len(self._options)-1
                self._text_objects[self._active].set_color(self._colors[1])
            else:
                self._text_objects[self._active].set_color(self._colors[0])
                self._active -= 1
                self._text_objects[self._active].set_color(self._colors[1])
        elif symbol == wkey.DOWN:
            if self._active >= len(self._options)-1:
                self._text_objects[self._active].set_color(self._colors[0])
                self._active = 0
                self._text_objects[self._active].set_color(self._colors[1])
            else:
                self._text_objects[self._active].set_color(self._colors[0])
                self._active += 1
                self._text_objects[self._active].set_color(self._colors[1])

        return True



"""The setup for 'stargateRL' python sci-fi rouge-like game.

See:
https://github.com/thee-engineer/stargateRL
"""

import io

from setuptools import setup, find_packages
from codecs import open as copen
from os import path

CWD = path.abspath(path.dirname(__file__))

with copen(path.join(CWD, 'README.rst'), encoding='utf-8') as f:
    LONG_DESCRIPTION = f.read()
    LONG_DESCRIPTION = LONG_DESCRIPTION.replace("\r", "")


def read(*names, **kwargs):
    """Read file contents."""
    with io.open(
        path.join(path.dirname(__file__), *names),
        encoding=kwargs.get("encoding", "utf8")
    ) as fp:
        return fp.read()


setup(
    name='stargateRL',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.0.2',

    description='A sci-fi rouge-like game, developed in Python using Pyglet.',
    long_description=LONG_DESCRIPTION,

    # The project's main homepage.
    url='https://github.com/thee-engineer/stargateRL',

    # Author details
    author='Alexandru-Paul Copil (thee-engineer)',
    author_email='alexandru.p.copil@gmail.com',

    # Choose your license
    license='Modified Simple Machine License',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project?
        'Development Status :: 1 - Planning',

        # Indicate who your project is intended for
        'Intended Audience :: Other Audience',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',

        # Topics
        'Topic :: Games/Entertainment',
        'Topic :: Multimedia :: Graphics',

        # Environment
        'Environment :: X11 Applications',
        'Environment :: MacOS X',
        'Environment :: Win32 (MS Windows)',

        # Pick your license as you wish (should match "license" above)
        'License :: Other/Proprietary License',

        # Language
        'Natural Language :: English',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
    ],

    # What does your project relate to?
    keywords='stargateRL stargate scifi sci-fi rouge-like rougelike game',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['docs', 'old', 'tests', 'dist', 'build']),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    data_files=[('config', ['config.json']),
                ('bins', ['bin/icon.png', 'bin/tileset.png'])],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'stargateRL = stargateRL.launcher.__main__',
        ],
    },
)
